/* GTS - Library for the manipulation of triangulated surfaces
 * Copyright (C) 1999 Stphane Popinet
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "gts.h"

static void func (gdouble ** f, GtsCartesianGrid g, guint k, gpointer data)
{
  gdouble x, y, z = g.z;
  guint i, j;

  for (i = 0, x = g.x; i < g.nx; i++, x += g.dx)
    for (j = 0, y = g.y; j < g.ny; j++, y += g.dy)
      f[i][j] = x*x + y*y + z*z;
}

/* Returns isosurface f(x, y, z) = iso with f(x, y, z) = x*x + y*y + z*z. */
int main (int argc, char * argv[])
{
  GtsCartesianGrid g;
  GtsSurface * surface;
  gdouble iso;

  if (argc != 5) {
    fprintf (stderr, "usage: iso nx ny nz iso\n");
    return 1;
  }
  
  iso = atof (argv[4]);
  g.nx = atoi (argv[1]);
  g.ny = atoi (argv[2]);
  g.nz = atoi (argv[3]);
  /* interval is [-1:1][-1:1][-1:1] */
  g.x = -1.0; g.dx = 2./(gdouble) (g.nx - 1);
  g.y = -1.0; g.dy = 2./(gdouble) (g.ny - 1);
  g.z = -1.0; g.dz = 2./(gdouble) (g.nz - 1);

  surface = gts_surface_new (gts_surface_class (),
			     gts_face_class (),
			     gts_edge_class (),
			     gts_vertex_class ());
  gts_isosurface_cartesian (surface, g, func, NULL, iso);
  
  gts_surface_print_stats (surface, stderr);
  gts_surface_write (surface, stdout);

  return 0;
}
